package examples;

import gui.TreeViewer;

import java.util.Arrays;

import sdf.SDF;
import sdf.SDF_Variable;


import nenok.Architecture;
import nenok.LCFactory;
import nenok.Services;
import nenok.lc.JoinTree;
import nenok.net.Knowledgebase;
import nenok.net.Locator;
import nenok.net.processor.Processor;
import nenok.net.registry.Registry;
import nenok.va.Domain;
import nenok.va.Valuation;
import nenok.va.Variable;

/**
 * 
 * 
 * @author Antoine De Groote
 */

public class Small {
    
    // The name of the knowledgebase:
    private static final String NAME = "Distances";

    /**
     * @param args
     */
    
    public static void main(String[] args) {
        
        // Verify number of arguments:
        if(args.length != 3) {
            System.out.println("Wrong number of Arguments.");
            System.out.println("Usage: Small #proc (Interactive | Console) (Local | Remote)");
            System.exit(1);
        }
        
        // Fetch number of processors:
        int procs = Integer.parseInt(args[0]);
        if(procs < 1) {
            System.out.println("The number of processors should be positive.");
            System.out.println("Usage: Small #proc (Interactive | Console) (Local | Remote)");
            System.exit(1);
        }
        
        /*
         * Variables:
         */
        
        SDF_Variable a = new SDF_Variable("A");
        SDF_Variable b = new SDF_Variable("B");
        SDF_Variable c = new SDF_Variable("C");
        SDF_Variable d = new SDF_Variable("D");
        SDF_Variable e = new SDF_Variable("E");
        
        /*
         * Valuations:
         */
        
        double inf = Double.POSITIVE_INFINITY;
            
    	double[][] dists1 = new double[][] { 
    			{ 0,3,2,inf },
    			{ 7,0,inf,1 },
    			{ 3,10,0,inf },
    			{ inf,inf,2,0 }
    	};
        SDF phi = new SDF(new SDF_Variable[] { a,b,c,d }, dists1);
        
        double[][] dists2 = new double[][] { 
        		{ 0,inf,3 },
        		{ 2,0,inf },
        		{ 5,1,0 }
        };
        SDF psi = new SDF(new SDF_Variable[] { c,d,e }, dists2);

        Valuation[] sdfs = new SDF[] { phi, psi };
                
        /*
         * Queries:
         */
        
        Domain[] queries = new Domain[] {
            new Domain(),
            new Domain(new Variable[] { a }),
            new Domain(new Variable[] { c,e }),
            new Domain(new Variable[] { a,b,c,d }),
            new Domain(new Variable[] { a,b,c,d,e })
        };
        
        /*
         * Setup NENOK Federation:
         */
        
        if(!args[2].equalsIgnoreCase("Local") && !args[3].equalsIgnoreCase("Remote")) {
            System.out.println("Wrong third Argument.");
            System.out.println("Usage: Small #proc (Interactive | Console) (Local | Remote)");
            System.exit(1);
        }
        
        try {
            
            // Run Processors :
            Processor[] processors = new Processor[procs]; 
           
            for (int i = 0; i < procs; i++) {
                processors[i] = (args[2].equalsIgnoreCase("Local")) ? Services.startLocalProcessor() : Services.startRemoteProcessor();
            }
            
            Locator[] locators = new Locator[sdfs.length]; 
            
            // Store &amp; register potentials:
            for (int i = 0, j = 0; i < sdfs.length; i++, j++) {
                locators[i] = processors[j % procs].store(sdfs[i]);
            }
            
            Knowledgebase kb = null;
            if(args[2].equalsIgnoreCase("Remote")) {
            	
                // Create knowledgebase in Registry:
                Registry registry = Services.findRegistry();
                registry.createKnowledgebase(NAME);
                registry.addAll(NAME, locators);
                
                kb = registry.getKnowledgebase(NAME);
            	
            } else {
            	
            	// Create local knowledgebase object:
            	kb = new Knowledgebase(locators);
            	kb.setName(NAME);
            	
            }
            
            /*
             * Local Computation:
             */
            
            if(args[1].equalsIgnoreCase("Console")) {
            
                LCFactory factory = new LCFactory();
                factory.setArchitecture(Architecture.Idempotent_Architecture);
                JoinTree tree = factory.create(kb, Arrays.asList(queries), processors[0]);
                tree.propagate();
                
                /*
                 * Query answering:
                 */
                
                System.out.println("Marginals of the Adder Example:\n");
                
                for(Domain query : queries) {
                    System.out.println(tree.answer(query));
                }
                            
            } else if(args[1].equalsIgnoreCase("Interactive")) {
            	            	
            	if(args[2].equalsIgnoreCase("Remote"))
            		TreeViewer.display();
            	TreeViewer.display(kb);
                
            } else {
                System.out.println("Wrong second Argument.");
                System.out.println("Usage: Small #proc (Interactive | Console) (Local | Remote)");
            }
            
            /*
             * Destroy Processors:
             */
            
            if(args[2].equalsIgnoreCase("Remote")) {
            	Services.findRegistry().deleteKnowledgebase(NAME);
            }
            
            Services.destroyAllProcessors();

        } catch (Exception e1) {
            e1.printStackTrace();
            System.exit(1);
        }
    }
}
