package sdf;

import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import nenok.parser.ParserException;
import nenok.parser.XmlParser;
import nenok.va.Valuation;
import nenok.va.Variable;

import org.jdom.Element;

/**
 * This class provides a XML parser implementation for distance functions.
 * XML Schema: sdf.xsd
 * 
 * @author Antoine De Groote
 */

public class SDF_Parser extends XmlParser {
	
	/**
	 * @see nenok.parser.XmlParser#parseVariable(java.lang.String, java.util.List)
	 */
	
	public Variable parseVariable(String name, List<Element> content) throws ParserException {
		return new SDF_Variable(name);
	} 
	
	/**
	 * @see nenok.parser.XmlParser#parseValuation(java.util.List, java.util.Hashtable)
	 */
	
	public Valuation parseValuation(List<Element> content, Hashtable<String, Variable> variables) throws ParserException {
		Vector<String> cities = new Vector<String>();
		Vector<Double> dists = new Vector<Double>();
		Hashtable<SDF_Variable, Hashtable<SDF_Variable, Double>> values = new Hashtable<SDF_Variable, Hashtable<SDF_Variable, Double>>(); 
		
		for (Element elem : content) {
			if (elem.getName().equals("label")) {
				continue;
			}
			else if (elem.getName().equals("cities")) {

				Element el = null;
				String city = null;
				Double dist = null;
				for (Object e : elem.getChildren()) {
					el = (Element) e;
					city = el.getText();
					dist = Double.parseDouble(el.getAttributeValue("distance"));
					cities.addElement(city);
					dists.addElement(dist);
				}
				
				for (int i = 0; i < cities.size(); i++) {
					values.put((SDF_Variable)variables.get(cities.get(i)), new Hashtable<SDF_Variable, Double>());
				}

				// first initialize all elements to infinity (resp. 0 on diagonal), so that all values are filled in.
				// Create a hashtable for each variable in the new domain, 
				// with values initialized to inifinity (POSITIVE_INFINITY).
				for (int i = 0; i < cities.size(); i++) {
					Hashtable<SDF_Variable, Double> cur = new Hashtable<SDF_Variable, Double>();
					SDF_Variable var = (SDF_Variable) variables.get(cities.get(i));
					for (SDF_Variable var2 : values.keySet()) {
						if (var.equals(var2)) {
							cur.put(var2, 0.0);
						} else {
							cur.put(var2, Double.POSITIVE_INFINITY);
						}
					}
					values.put(var, cur);
				}
				
				// now fill in (overwrite) the known values
				for (int i = 0; i < dists.size(); i++) {
					SDF_Variable var0 = (SDF_Variable) variables.get(cities.get(0));
					SDF_Variable vari = (SDF_Variable) variables.get(cities.get(i));
					values.get(var0).put(vari, dists.get(i));
					values.get(vari).put(var0, dists.get(i));
				}
			} else {
				throw new IllegalArgumentException(elem.getName());
			}
		}
		SDF_Variable[] vars = new SDF_Variable[cities.size()];
		for (int i = 0; i < cities.size(); i++) {
			vars[i] = (SDF_Variable) variables.get(cities.get(i));
		}
		
		return new SDF(vars, values);
	}   
	
}
