package indicators;

import nenok.va.semiring.FiniteVariable;

/**
 * This implementation of the variable interface represents 
 * variables as simple strings. The frame of each variable is 
 * binary, i.e. frame = {0,1}.
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public class BinaryVariable implements FiniteVariable {

    private final String name;
    private static final String[] frame = new String[] {"0", "1"};
    
	/**
	 * Constructor:
	 * @param name The name of the variable.	 
	 */
	
	public BinaryVariable(String name) {
		this.name = name;
	}
		
	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	
	public boolean equals(Object obj) {
		return this == obj || (getClass() == obj.getClass() && ((BinaryVariable)obj).name.equals(name));
	}
	
	/**
     * @see java.lang.Object#hashCode()
     */

    public int hashCode() {
        return name.hashCode();
    }

    /**
	 * @see java.lang.Object#toString()
	 */

	public String toString() {
		return name;
	}
    
	/**
     * @see nenok.va.semiring.FiniteVariable#getFrame()
     */
	
    public Object[] getFrame() {
        return frame;
    }
}