package ppotentials;

import nenok.va.Domain;
import nenok.va.Predictor;
import nenok.va.Variable;

/**
 * A weight predictor for probability potentials implemented as
 * a SINGLETON design pattern.
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public class PP_Predictor implements Predictor {
    
    private static PP_Predictor self;

    /**
     * Constructor:
     */
    
    private PP_Predictor() {}
    
    /**
     * @return An instance of this class - SINGLETON design pattern.
     */
    
    public static PP_Predictor getInstance() {
        if(self == null) {
            self = new PP_Predictor();
        }
        return self;
    }
    
    /**
     * @see nenok.va.Predictor#predict(nenok.va.Domain)
     */
    
    public int predict(Domain dom) {
        if(dom.size() == 0) {
            return 0;
        }
        int result = 1;
        for(Variable var : dom) {
            PP_Variable v = (PP_Variable)var;
            result *= v.getFrame().length;
        }
        return result;
    }
}
