package ppotentials;

import nenok.va.semiring.FiniteVariable;

/**
 * This implementation of the DF_Variable interface represents 
 * variables as simple strings. Each variable knows its frame, 
 * reprsenting the variable's possible values.
 * 
 * @author Marc Pouly
 * @version 1.0
 */

public class PP_Variable implements FiniteVariable {

	private String name;
	private String[] frame;
	
	/**
	 * Constructor:
	 * @param name The name of the variable.
	 * @param frame The variable's possible values.	 
	 */
	
	public PP_Variable(String name, String[] frame) {
		this.name = name;
		this.frame = frame;
	}
		
	/** 
     * @see nenok.va.semiring.FiniteVariable#getFrame()
	 */
	
	public Object[] getFrame() {
		return this.frame;
	}
		
	/**
	 * @see java.lang.Object#hashCode()
	 */
		
	public int hashCode() {
		return name.hashCode();
	}
	
	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	
	public boolean equals(Object o) {
	    return (this == o) || ((o instanceof PP_Variable) && 
	    ((PP_Variable)o).name.equals(this.name));
	}
	
	/**
	 * Converts the current variable to a string.
	 * @return The variable's string value.
	 */

	public String toString() {
		return name;
	}
}