package ppotentials;

import nenok.va.semiring.RegularElement;
import nenok.va.semiring.Element;

/**
 * This class represents probabilities as dividable semiring element. 
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public class Probability implements RegularElement {
            
    private double value;
    
    //Distance for equality checks:
    private static final double EPSILON = 0.0000001;
        
    /**
     * Constructor:
     * @param value The probability value.
     */
    
    public Probability(double value) {
        this.value = value;
    }

    /**
     * @see nenok.va.semiring.Element#add(nenok.va.semiring.Element)
     */
    
    public Element add(Element element) {
        if(!(element instanceof Probability)) {
            throw new IllegalArgumentException(element+" not instance of class "+Probability.class+".");
        }
        return new Probability(value + ((Probability)element).value);
    }

    /**
     * @see nenok.va.semiring.Element#multiply(nenok.va.semiring.Element)
     */
    
    public Element multiply(Element element) {
        if(!(element instanceof Probability)) {
            throw new IllegalArgumentException(element+" not instance of class "+Probability.class+".");
        }
        return new Probability(value * ((Probability)element).value);
    }
    
    /**
     * @see nenok.va.semiring.RegularElement#inverse()
     */

    public RegularElement inverse() {
        return (this.value == 0)? new Probability(0) : new Probability(1 / value);
    }    
    
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    
    public boolean equals(Object o) {
        if(!(o instanceof Probability)) {
            return false;
        }
        return numericTest(((Probability)o).value, value);
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    
    public String toString() {
        return String.format("%.2f", value);
    }

    /*
     * Getter &amp; Setter:
     */
    
    /**
     * @return The probability object's value.
     */
    
    protected double getValue() {
        return this.value;
    }
    
    /**
     * This test compares two numeric double values on equality.
     * They are equal if their distance is smaller that the EPSILON constant.
     * @param d1 The first value.
     * @param d2 The second value.
     * @return <code>true</code>, if the two values are equal.
     */
    
    protected static boolean numericTest(double d1, double d2) {
        double distance = Math.abs(d1 - d2);
        return !(distance > EPSILON);
    }
}
