package ppotentials;

import javax.swing.JPanel;

import nenok.va.semiring.Configuration;
import nenok.va.semiring.Element;
import nenok.va.semiring.ScaledSRValuation;

/**
 * Probability potential are an example of a scaled semiring induced valuation algebra 
 * and this class offers a possible implementation by extending the {@link ScaledSRValuation}
 * class. The variables used within this class are of type {@link PP_Variable}.
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public class SRPotential extends ScaledSRValuation {
    
	/*
	 * Constructors:
	 */
	
	/**
	 * Constructor:
	 */
	
	public SRPotential() {}
		
	/**
	 * Constructor:
	 * @param variable The main variable of this potential. 
	 * @param probabilities The probability for each possible configuration.
	 */
	
	public SRPotential(PP_Variable variable, double[] probabilities) {
		super(new PP_Variable[] {variable}, transform(probabilities));
	}
		
	/**
	 * Constructor:
	 * @param variable The main variable of this potential. 
	 * @param conditionals The conditional variables of this potential.
	 * @param probabilities The probability for each possible configuration.
	 */	
		
	public SRPotential(PP_Variable variable, PP_Variable[] conditionals, double[] probabilities) {
		super(addToArray(variable, conditionals), transform(probabilities));
	}
	
	/*
	 * Static helper methods:
	 */
	
	/**
	 * Object converter:
	 * @param probabilities An array of probability values.
	 * @return An array of {@link Probability} objects built from the given double values.
	 */
	
	private static Probability[] transform(double[] probabilities) {
        Probability[] elements = new Probability[probabilities.length];
		for (int i = 0; i < elements.length; i++) {
            elements[i] = new Probability(probabilities[i]);
        }
		return elements;
	}
	
	/**
	 * Adds a variables to an existing variable array.
	 * @param v The variable to add.
	 * @param vars The existing variable array.
	 * @return The variable array built from the arguments.
	 */
	
	private static PP_Variable[] addToArray(PP_Variable v, PP_Variable[] vars) {
	    PP_Variable[] variables = new PP_Variable[vars.length + 1];
        variables[0] = v;
		for (int i = 1; i < variables.length; i++) {
			variables[i] = vars[i-1];
		}
		return variables;
	}
	
	/*
	 * Output method:
	 */
	
    /**
     * Displays the current potential as a chart.
     * @return The current potential as a chart.
     */
    
    public JPanel displayChart() {
        return new Chart(this);
    }
        
    /*
     * Comparison operator for testing purposes:
     */
    
    /**
     * Comparaison between the two implementation of probability potentials.
     * @param p The probability potential to compare.
     * @return <code>true</code>, if the two objects are equal by value.
     */
    
    public boolean equals(Potential p) {
        if(!p.label().equals(this.label())) {
            return false;
        }
        if(p.getProbabilities().length != this.elements.length) {
        	return false;
        }
        for (int i = 0; i < p.getProbabilities().length; i++) {
            if(!Probability.numericTest(p.getProbabilities()[i], ((Probability)(this.elements[i])).getValue())) {
            	return false;
            }
        }
        return true;
    }
    
    /*
     * Getter: 
     */
    
    /**
     * @return The array of semiring values that belongs to this potential.
     */
    
    public Element[] getElements() {
    	return this.elements;
    }
    
    /**
     * @return The array of configurations within this potential.
     */
    
    protected Configuration[] getConfigurations() {
    	return this.configurations;
    }
}
