package nenok.lc.alloc;

import java.rmi.RemoteException;

import nenok.lc.Node;

/**
 * This is the basic interface for the node processor allocation strategy. 
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public interface Allocator {
    
    /**
     * Dynamic allocation procedure. Allocates a processor to the node given as argument.
     * This method is called from the collect algorithm for each node strictly before any
     * computations are executed onto this node. Note: Static allocation is not possible 
     * with this method. To do so, we recomment to implement a corresponding constructor
     * and to provide an empty implementation of this method.
     * @param node The current node to which a processor needs to be allocated.
     * @throws RemoteException Communication exception caused by accessing processors.
     */
    
    public void allocate(Node node) throws RemoteException;

    /**
     * @return Returns the total communication costs caused so far.
     */
    
    public int totalCosts();
    
    /**
     * @return The name of this processor allocation model.
     */
    
    public String getName();
}
