package nenok.lc.constr;

import java.io.File;
import java.rmi.RemoteException;

import nenok.lc.Node;
import nenok.lc.JoinTree.Construction;
import nenok.parser.JT_Parser;
import nenok.parser.ParserException;

/**
 * This construction algorithm allows to read and rebuild 
 * jointrees that were serialized to a file.
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public class Deserializer extends Algorithm {
    
    private File file;
    private JT_Parser parser;

    /**
     * Constructor:
     * @param file The file that contains the serialized jointree.
     * @param parser The parser to deserialize the given file.
     */
    
    public Deserializer(File file, JT_Parser parser) {
        this.file = file;
        this.parser = parser;
    }
    
    /**
     * @see nenok.lc.constr.Algorithm#buildTree(nenok.lc.JoinTree.Construction)
     */

    public Node buildTree(Construction data) throws ConstrException, RemoteException {
        try {
        	return parser.parse(file, data);
        } catch (ParserException e) {
            throw new ConstrException(e);
        }
    }
    
    /**
     * @see nenok.lc.constr.Algorithm#getName()
     */

    public String getName() {
        return "File Deserializer";
    }
}
