package nenok.lc.id;

import java.rmi.RemoteException;
import java.util.Collection;

import nenok.Utilities;
import nenok.lc.JoinTree;
import nenok.lc.Node;
import nenok.lc.constr.Algorithm;
import nenok.lc.constr.ConstrException;
import nenok.net.Knowledgebase;
import nenok.net.processor.Processor;
import nenok.va.Domain;
import nenok.va.Idempotency;
import nenok.va.VAException;

/**
 * The join tree corresponding to the idempotent architecture.
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public class IDJoinTree extends JoinTree {
	    
	/**
	 * Constructor:
     * @param kb The knowledgebase covered by this join tree.
     * @param queries The set of queries that are covered by this join tree.
     * @param algo The construction algorithm.
     * @param proc The processor assigned to query nodes.
     * @throws ConstrException Exceptions throws during construction process.
     * @throws RemoteException Exceptions caused by communication problems when accessing the query processor.
	 */
	
	public IDJoinTree(Knowledgebase kb, Collection<Domain> queries, Algorithm algo, Processor proc) throws ConstrException, RemoteException {
        super(kb, queries, algo, proc);
    }

    /** 
     * @see nenok.lc.JoinTree#verify(java.lang.Class)
     */
    
    public boolean verify(Class type) {
        return Utilities.isInterfaceOf(Idempotency.class, type);
    }

    /** 
     * @see nenok.lc.JoinTree#getArchitecture()
	 */
	 
	public String getArchitecture() {
		return "Idempotent";
	}

	/** 
     * @see nenok.lc.JoinTree#scale()
	 */
	
	public void scale() throws VAException, RemoteException {
        // scaling is redundant in idempotent architecture
	}

	/**
     * @see nenok.lc.JoinTree#createNode()
	 */
    
	public Node createNode() {
        return new IDNode(factory);
	}
}
