package nenok.lc.ss;

import java.rmi.RemoteException;
import java.util.Collection;

import nenok.lc.constr.Algorithm;
import nenok.lc.constr.ConstrException;
import nenok.net.Knowledgebase;
import nenok.net.processor.Processor;
import nenok.va.Domain;

/**
 * The join tree corresponding to the binary Shenoy-Shafer architecture.
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public class BSSJoinTree extends SSJoinTree {
	    
	/**
	 * Constructor:
     * @param kb The knowledgebase covered by this join tree.
     * @param queries The set of queries that are covered by this join tree.
     * @param algo The construction algorithm.
     * @param proc The processor assigned to query nodes.
     * @throws ConstrException Exceptions throws during construction process.
     * @throws RemoteException Exceptions caused by communication problems when accessing the query processor.
	 */
	
	public BSSJoinTree(Knowledgebase kb, Collection<Domain> queries, Algorithm algo, Processor proc) throws ConstrException, RemoteException {
        super(kb, queries, algo, proc);
        double start = System.currentTimeMillis();
        makeBinary();
        double time = System.currentTimeMillis() - start;
        algo.setConstructionTime(algo.getConstructionTime() + time);
    }

	/** 
     * @see nenok.lc.JoinTree#getArchitecture()
	 */
	 
	public String getArchitecture() {
		return "Binary Shenoy-Shafer";
	}
}
