package nenok.lc.ss;

import java.rmi.RemoteException;
import java.util.Collection;

import nenok.lc.JoinTree;
import nenok.lc.Node;
import nenok.lc.constr.Algorithm;
import nenok.lc.constr.ConstrException;
import nenok.net.Knowledgebase;
import nenok.net.Locator;
import nenok.net.processor.Processor;
import nenok.net.task.Task;
import nenok.va.Domain;
import nenok.va.VAException;

/**
 * The join tree corresponding to the n-ary Shenoy-Shafer architecture.
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public class SSJoinTree extends JoinTree {
	    
	/**
	 * Constructor:
     * @param kb The knowledgebase covered by this join tree.
     * @param queries The set of queries that are covered by this join tree.
     * @param algo The construction algorithm.
     * @param proc The processor assigned to query nodes.
     * @throws ConstrException Exceptions throws during construction process.
     * @throws RemoteException Exceptions caused by communication problems when accessing the query processor.
	 */
	
	public SSJoinTree(Knowledgebase kb, Collection<Domain> queries, Algorithm algo, Processor proc) throws ConstrException, RemoteException {
        super(kb, queries, algo, proc);
    }

	/** 
     * @see nenok.lc.JoinTree#verify(java.lang.Class)
	 */
    
    public boolean verify(Class type) {
        return true;
    }

    /** 
     * @see nenok.lc.JoinTree#getArchitecture()
	 */
	 
	public String getArchitecture() {
		return "Shenoy-Shafer";
	}

	/** 
     * @see nenok.lc.JoinTree#scale()
	 */
	
	public void scale() throws VAException, RemoteException {
            
        /*
         * Compute inverse (scaling factor):
         */
		
        Processor p = root.getProcessor();
        Locator result = root.getLocator();
        
        if(result.getDomain().size() > 0) {
            Task marg = factory.createMarginalization(root.getLocator(), Domain.EMPTY);
            result = p.execute(marg);
        }
        
        Task inv = factory.createInverse(result);
        result = p.execute(inv);
		        
        /*
         * Send scaling factor to root:
         */

        ((SSNode)getRoot()).setScalingFactor(result);
        
        /*
         * Update root content:
         */
        
        Task comb = factory.createCombination(root.getLocator(), result);
        this.getRoot().setLocator(p.execute(comb));
	}

	/**
     * @see nenok.lc.JoinTree#createNode()
	 */
    
	public Node createNode() {
        return new SSNode(factory);
	}
}
