package nenok.net;

import java.rmi.RemoteException;

import nenok.Services;
import nenok.net.processor.Processor;
import nenok.va.Domain;
import nenok.va.Valuation;
import net.jini.id.Uuid;

/**
 * Default implementation of the {@link Locator} interface.
 * 
 * @author Marc Pouly
 * @version 1.0
 */

public class DefaultLocator extends Locator {
	
	private final Uuid envID;
	private final Domain dom;
	private final String type;
	private final int weight;

	/**
	 * Constructor:
	 * @param pid The identifier of the processor hosting the valuation.
	 * @param env The wrapper envelope of the valuation.
	 */

	public DefaultLocator(Uuid pid, Envelope env) {
		super(pid);
		this.envID = env.getID();
		this.dom = env.getValuation().label();
        this.weight = env.getValuation().weight();
		this.type = env.getValuation().getClass().getName();
	}

	/**
	 * @see Locator#retrieve()
	 */
	
	public Valuation retrieve() throws RemoteException {
		Processor p = Services.findProcessor(this);
		return (p != null)? p.retrieve(this) : null;
	}
	
	/**
	 * @see Locator#delete()
	 */
	
	public void delete() throws RemoteException {
		Processor p = Services.findProcessor(this);
		if(p != null) {
			p.delete(this);
		}
	}
	
	/**
	 * @see Locator#getDomain()
	 */
	
    public Domain getDomain() {
        return dom;
    } 
    
    /**
     * @return The weight of the referenced valuation.
     */
    
    public int getWeight() {
        return weight;
    } 
    
	/**
	 * @see Locator#getType()
	 */
    	
    public String getType() {
    	return this.type;
    }
	
	/**
	 * @return The identifier of the envelope that wraps the referenced valuation.
	 */
    
    public Uuid getEnvelopeID() {
    	return envID;
    }
	
    /**
     * @see java.lang.Object#hashCode()
     */

    public int hashCode() {
    	return envID.hashCode();
    }
    
    /**
     * @see java.lang.Object#toString()
     */

    public String toString() {
    	return "(Envelope: "+getEnvelopeID()+", Processor: "+getPID()+")";
    }
    
    /**
     * Two locators are equal, if they point to the same object.
     * @see java.lang.Object#equals(java.lang.Object)
     */

    public boolean equals(Object o) {
    	if(!(o instanceof DefaultLocator)) {
    		return false;
    	}
    	DefaultLocator loc = (DefaultLocator)o;
    	return loc.getPID().equals(getPID()) && loc.envID.equals(envID);
    }
}
