package nenok.net;

import java.rmi.RemoteException;

import nenok.va.Domain;
import nenok.va.Identity;
import nenok.va.Valuation;
import net.jini.id.Uuid;

/**
 * Implementation of the {@link Locator} interface for identity elements.
 * 
 * @author Marc Pouly
 * @version 1.2
 */

public class IdentityLocator extends Locator {
		
	/**
	 * Constructor:
     * @param pid The identifier of the processor hosting the valuation.
	 */
	
	public IdentityLocator(Uuid pid) {
		super(pid);
	}

	/**
	 * @see nenok.net.Locator#delete()
	 */
	
	@Override
	public void delete() throws RemoteException {
		// Do nothing ...
	}

	/**
	 * @see nenok.net.Locator#getDomain()
	 */
	
	@Override
	public Domain getDomain() {
		return Identity.INSTANCE.label();
	}

	/**
	 * @see nenok.net.Locator#getType()
	 */
	
	@Override
	public String getType() {
		return Identity.class.getName();
	}

	/**
	 * @see nenok.net.Locator#getWeight()
	 */
	
	@Override
	public int getWeight() {
		return Identity.INSTANCE.weight();
	}

	/**
	 * @see nenok.net.Locator#retrieve()
	 */
	
	@Override
	public Valuation retrieve() {
		return Identity.INSTANCE;
	}
	
	/**
	 * @see java.lang.Object#equals(Object)
	 */
	
	@Override
	public boolean equals(Object obj) {
		if(!(obj instanceof IdentityLocator)) {
			return false;
		}
		IdentityLocator il = (IdentityLocator)obj;
		return this.getPID().equals(il.getPID());
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	
	@Override
	public String toString() {
		return "Identity Locator: PID = "+getPID();
	}
}
