package nenok.net.task;

import java.rmi.RemoteException;

import nenok.net.Envelope;
import nenok.net.Locator;
import nenok.va.Identity;
import nenok.va.Valuation;

/**
 * Command design pattern class for the combination operation.
 * 
 * @author Marc Pouly
 * @version 1.2
 */

public final class Combination implements Task {

	private Locator[] factors;
    
    /**
     * Constructor:
     * @param factors An array of factors to combine.
     */
    
    protected Combination(Locator[] factors) {
        this.factors = factors;
    }
        
    /**
     * @see nenok.net.task.Task#execute()
     */
    
    public Envelope execute() throws RemoteException {
    	
    	if(factors.length == 0) {
    		return new Envelope(Identity.INSTANCE);
    	}
                                
        /*
         * Fetch operation's arguments & execute task:
         */
    	
    	Valuation result = factors[0].retrieve();
    	for(int i = 1; i < factors.length; i++) {
    		result = result.combine(factors[i].retrieve());
    	}

        return new Envelope(result);
  }

    /**
     * @see java.lang.Object#toString()
     */
    
    public String toString() {
        return "Combination";
    }
}
