package nenok.net.task;

import java.rmi.RemoteException;

import nenok.net.Envelope;
import nenok.net.Locator;
import nenok.va.Regularity;
import nenok.va.VAException;
import nenok.va.Valuation;

/**
 * Command design pattern class for the division operation.
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public final class Inverse implements Task {

    private Locator factor;
    
    /**
     * Constructor:
     * @param factor The factor whose inverse is computed.
     */
    
    protected Inverse(Locator factor) {
        this.factor = factor;
    }
        
    /**
     * @see nenok.net.task.Task#execute()
     */

    public Envelope execute() throws RemoteException, VAException {

        /*
         * Fetch operation's arguments:
         */
        
        Regularity val = (Regularity)factor.retrieve();
                                                     
        /*
         * Execute operation:
         */
      
        Valuation r = val.inverse();  
        return new Envelope(r);
    }

    /**
     * @see java.lang.Object#toString()
     */
    
    public String toString() {
        return "Inverse";
    }
}
