package nenok.net.task;

import java.rmi.RemoteException;

import nenok.net.Envelope;
import nenok.net.Locator;
import nenok.va.Domain;
import nenok.va.VAException;
import nenok.va.Valuation;

/**
 * Command design pattern class for the marginalization operation.
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public final class Marginalization implements Task {
    
    private Domain dom;
    private Locator factor;

    /**
     * Constructor:
     * @param factor The locator of the valuation to be marginalized.
     * @param dom The domain onto the marginalization is performed.
     */
    
    protected Marginalization(Locator factor, Domain dom) {
        this.factor = factor;
        this.dom = dom;
    }
    
    /**
     * @see nenok.net.task.Task#execute()
     */
    
    public Envelope execute() throws RemoteException, VAException {
  
        /*
         * Fetch operation's arguments:
         */
        
        Valuation val = factor.retrieve();
                                                                
        /*
         * Execute operation:
         */
        
        Valuation r = val.marginalize(dom);
        return new Envelope(r);
    }

    /**
     * @see java.lang.Object#toString()
     */
    
    public String toString() {
        return "Marginalization";
    }
}
