package nenok.net.task;

import java.rmi.RemoteException;

import nenok.net.Envelope;
import nenok.net.Locator;
import nenok.va.Scalability;
import nenok.va.VAException;
import nenok.va.Valuation;

/**
 * Command design pattern class for the scaling operation.
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public final class Scale implements Task {
    
    private Locator factor;
    
    /**
     * Constructor:
     * @param factor The factor whose scale is computed.
     */
    
    public Scale(Locator factor) {
        this.factor = factor;
    }
    
    /**
     * @see nenok.net.task.Task#execute()
     */

    public Envelope execute() throws RemoteException, VAException {
       
        /*
         * Fetch operation's arguments:
         */
        
        Scalability val = (Scalability)factor.retrieve();
                                         
        /*
         * Execute operation:
         */
                
        Valuation r = val.scale();
        return new Envelope(r);
    }

    /**
     * @see java.lang.Object#toString()
     */
    
    public String toString() {
        return "Scaling";
    }
}
