  package nenok.net.task;

import java.io.Serializable;
import java.rmi.RemoteException;

import nenok.net.Envelope;
import nenok.va.VAException;

/**
 * Head interface of a COMMAND design pattern implementation. Tasks encapsulate valuation algebra operations. They are
 * transmitted over a network to be executed afterwards on a remote NENOK processor.
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public interface Task extends Serializable {
                           
    /**
     * Execution method of the COMMAND design pattern.
     * @return The envelope containing the task's result.
     * @throws RemoteException Exception caused when remote processors are accessed.
     * @throws VAException Exception caused by illegal valuation algebra operations.
     */
    
    public Envelope execute() throws RemoteException, VAException;
}
