package nenok.net.task;

import nenok.net.Locator;
import nenok.va.Domain;

/**
 * A actory class to construct {@link Task} object. The main purpose of this 
 * class is to protocol the number of constructed objects.
 * 
 * @author Marc Pouly
 * @version 1.2
 */

public class TaskFactory {
    
    private int combinations, marginalizations, inverses, scales;
        
    /*
     * Factory methods:
     */
    
    /**
     * @param factors The factors to be combined.
     * @return A new combination task.
     */
    
    public Task createCombination(Locator... factors) {
        combinations += (factors.length == 0)? 0 : factors.length - 1;
        return new Combination(factors);
    }
    
    /**
     * @param factor The locator of the valuation to be marginalized.
     * @param dom The domain onto the marginalization is performed.
     * @return A new marginalization task.
     */
    
    public Task createMarginalization(Locator factor, Domain dom) {
        marginalizations++;
        return new Marginalization(factor, dom);
    }
    
    /**
     * @param factor The factor whose inverse is computed.
     * @return A new inverse task.
     */
    
    public Task createInverse(Locator factor) {
        inverses++;
        return new Inverse(factor);
    }
    
    /**
     * @param factor The factor whose scale is computed.
     * @return A new scaling task.
     */
    
    public Task createScale(Locator factor) {
        scales++;
        return new Scale(factor);
    }

    /*
     * Getters:
     */
    
    /**
     * @return Returns the number of combinations.
     */
    
    public int getCombinations() {
        return combinations;
    }
    
    /**
     * @return Returns the number of inverses.
     */
    
    public int getInverses() {
        return inverses;
    }
    
    /**
     * @return Returns the number of marginalizations.
     */
    
    public int getMarginalizations() {
        return marginalizations;
    }
    
    /**
     * @return Returns the number of scales.
     */
    
    public int getScales() {
        return scales;
    }    
}
