package nenok.va;

/**
 * Specifies an idempotent valuation. Idempotency is basically a trivial division, 
 * therefore this interface extends {@link Regularity} and provides furthermore 
 * a pre-implementation of the division operator. However, to prevent the user to be 
 * reliant on abstract classes (because multiple inheritance is not allowed in JAVA), 
 * the interface offers a delegator inner class that implements division. This delegator
 * can be used as follows:
 * <code>
 * public Dividability inverse(Domain dom) throws VAException {
 *    return new Idempotency.Implementor().inverse(this, dom);
 * }
 * </code>
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public interface Idempotency extends Regularity {
    
    /*
     * Marker interface:
     */
    
    /**
     * Implementor of the Idempotency interface.
     * 
     * @author Marc Pouly
     * @version 1.1
     */
    
    public class Implementor {

        /**
         * Pre-implementation of the {@link Regularity} interface.
         * @param factor The factor whose inverse has to be computed.
         * @return The inverse of the factor.
         */
        
        public Regularity inverse(Idempotency factor) {
            return factor;
        }
    }
}


