package nenok.va.semiring;

import nenok.va.Domain;
import nenok.va.Predictor;
import nenok.va.Variable;

/**
 * A weight predictor for semiring induced valuation algebras 
 * implemented as a SINGLETON design pattern.
 * 
 * @author Marc Pouly
 * @version 1.1
 */

public class SRPredictor implements Predictor {
    
    private static SRPredictor self;

    /**
     * Constructor:
     */
    
    private SRPredictor() {}
    
    /**
     * @return An instance of this class - SINGLETON design pattern.
     */
    
    public static SRPredictor getInstance() {
        if(self == null) {
            self = new SRPredictor();
        }
        return self;
    }
    
    /**
     * @see nenok.va.Predictor#predict(nenok.va.Domain)
     */
    
    public int predict(Domain dom) {
        if(dom.size() == 0) {
            return 0;
        }
        int result = 1;
        for(Variable var : dom) {
            FiniteVariable v = (FiniteVariable)var;
            result *= v.getFrame().length;
        }
        return result;
    }
}
